@@ -1593,12 +1593,23 @@ spMsPGOcc <- function(occ.formula, det.formula, data, inits, priors,
         class(out.fit) <- "spMsPGOcc"
 
         # Predict occurrence at new sites. 
-	if (p.occ.re > 0) {X.0 <- cbind(X.0, X.re.0 + 1)}
+	# Get RE levels correct for when they aren't supplied at values starting at 1.
+	if (p.occ.re > 0) {
+	  tmp <- unlist(re.level.names)
+	  X.re.0 <- matrix(tmp[c(X.re.0 + 1)], nrow(X.re.0), ncol(X.re.0))
+	  colnames(X.re.0) <- x.re.names
+	}
+	if (p.occ.re > 0) {X.0 <- cbind(X.0, X.re.0)}
         out.pred <- predict.spMsPGOcc(out.fit, X.0, coords.0, verbose = FALSE)
 
         # Generate detection values
-	# Need to add 1 to X.p.re.0
-        if (p.det.re > 0) {X.p.0 <- cbind(X.p.0, X.p.re.0 + 1)}
+        # Generate detection values
+	if (p.det.re > 0) {
+	  tmp <- unlist(p.re.level.names)
+	  X.p.re.0 <- matrix(tmp[c(X.p.re.0 + 1)], nrow(X.p.re.0), ncol(X.p.re.0))
+	  colnames(X.p.re.0) <- x.p.re.names
+	}
+        if (p.det.re > 0) {X.p.0 <- cbind(X.p.0, X.p.re.0)}
         out.p.pred <- predict.spMsPGOcc(out.fit, X.p.0, type = 'detection')
 
         if (binom) {
@@ -2239,12 +2250,22 @@ spMsPGOcc <- function(occ.formula, det.formula, data, inits, priors,
         class(out.fit) <- "spMsPGOcc"
 
         # Predict occurrence at new sites. 
-	if (p.occ.re > 0) {X.0 <- cbind(X.0, X.re.0 + 1)}
+	if (p.occ.re > 0) {
+	  tmp <- unlist(re.level.names)
+	  X.re.0 <- matrix(tmp[c(X.re.0 + 1)], nrow(X.re.0), ncol(X.re.0))
+	  colnames(X.re.0) <- x.re.names
+	}
+	if (p.occ.re > 0) {X.0 <- cbind(X.0, X.re.0)}
         out.pred <- predict.spMsPGOcc(out.fit, X.0, coords.0, verbose = FALSE)
 
         # Generate detection values
-	# Need to add 1 to X.p.re.0
-        if (p.det.re > 0) {X.p.0 <- cbind(X.p.0, X.p.re.0 + 1)}
+        # Generate detection values
+	if (p.det.re > 0) {
+	  tmp <- unlist(p.re.level.names)
+	  X.p.re.0 <- matrix(tmp[c(X.p.re.0 + 1)], nrow(X.p.re.0), ncol(X.p.re.0))
+	  colnames(X.p.re.0) <- x.p.re.names
+	}
+        if (p.det.re > 0) {X.p.0 <- cbind(X.p.0, X.p.re.0)}
         out.p.pred <- predict.spMsPGOcc(out.fit, X.p.0, type = 'detection')
 
         if (binom) {
