@@ -1656,8 +1656,11 @@ svcTPGOcc <- function(occ.formula, det.formula, data, inits, priors,
 
 	# Predict occurrence at new sites
         if (p.occ.re > 0) {
-	  X.0 <- cbind(X.0, X.re.0 + 1)
-	}
+          tmp <- unlist(re.level.names)
+          X.re.0 <- matrix(tmp[c(X.re.0 + 1)], nrow(X.re.0), ncol(X.re.0))
+          colnames(X.re.0) <- x.re.names
+          X.0 <- cbind(X.0, X.re.0)
+        }
 	tmp.names <- colnames(X.0)
 	X.0 <- array(X.0, dim = c(J.0, n.years.max, ncol(X.0)))
 	dimnames(X.0)[[3]] <- tmp.names
@@ -1665,7 +1668,12 @@ svcTPGOcc <- function(occ.formula, det.formula, data, inits, priors,
 				      verbose = FALSE)
 
         # Predict detection values 
-        if (p.det.re > 0) {X.p.0 <- cbind(X.p.0, X.p.re.0 + 1)}
+        if (p.det.re > 0) {
+          tmp <- unlist(p.re.level.names)
+          X.p.re.0 <- matrix(tmp[c(X.p.re.0 + 1)], nrow(X.p.re.0), ncol(X.p.re.0))
+          colnames(X.p.re.0) <- x.p.re.names
+        }
+        if (p.det.re > 0) {X.p.0 <- cbind(X.p.0, X.p.re.0)}
         tmp.names <- colnames(X.p.0)
         X.p.0 <- array(X.p.0, dim = c(nrow(X.p.0), 1, ncol(X.p.0)))
         dimnames(X.p.0)[[3]] <- tmp.names
