@@ -38,7 +38,7 @@ extern "C" {
     /**********************************************************************
      * Initial constants
      * *******************************************************************/
-    int i, j, s, l, q, r, info, nProtect=0;
+    int i, j, s, l, ll, q, r, info, nProtect=0;
     const int inc = 1;
     const double one = 1.0;
     const double zero = 0.0;
@@ -150,7 +150,9 @@ extern "C" {
     int JN = J * N;
     int JpOcc = J * pOcc; 
     int nObspDet = nObs * pDet;
-    double tmp_0; 
+    int JpOccRE = J * pOccRE; 
+    int nObspDetRE = nObs * pDetRE;
+    double tmp_0, tmp_02; 
     double *tmp_one = (double *) R_alloc(inc, sizeof(double)); 
     double *tmp_ppDet = (double *) R_alloc(ppDet, sizeof(double));
     double *tmp_ppOcc = (double *) R_alloc(ppOcc, sizeof(double)); 
@@ -300,22 +302,26 @@ extern "C" {
     // Site-level sums of the occurrence random effects
     double *betaStarSites = (double *) R_alloc(JN, sizeof(double)); 
     zeros(betaStarSites, JN); 
+    int *betaStarLongIndx = (int *) R_alloc(JpOccRE, sizeof(int));
     // Initial sums (initiate with the first species)
-    for (i = 0; i < N; i++) {
-      for (j = 0; j < J; j++) {
-        for (l = 0; l < pOccRE; l++) {
-          betaStarSites[i * J + j] += betaStar[i * nOccRE + which(XRE[l * J + j], betaLevelIndx, nOccRE)];
+    for (j = 0; j < J; j++) {
+      for (l = 0; l < pOccRE; l++) {
+        betaStarLongIndx[l * J + j] = which(XRE[l * J + j], betaLevelIndx, nOccRE);
+        for (i = 0; i < N; i++) {
+          betaStarSites[i * J + j] += betaStar[i * nOccRE + betaStarLongIndx[l * J + j]];
         }
       }
     }
     // Observation-level sums of the detection random effects
     double *alphaStarObs = (double *) R_alloc(nObsN, sizeof(double)); 
     zeros(alphaStarObs, nObsN); 
+    int *alphaStarLongIndx = (int *) R_alloc(nObspDetRE, sizeof(int));
     // Get sums of the current REs for each site/visit combo for all species
-    for (i = 0; i < N; i++) {
-      for (r = 0; r < nObs; r++) {
-        for (l = 0; l < pDetRE; l++) {
-          alphaStarObs[i * nObs + r] += alphaStar[i * nDetRE + which(XpRE[l * nObs + r], alphaLevelIndx, nDetRE)];
+    for (r = 0; r < nObs; r++) {
+      for (l = 0; l < pDetRE; l++) {
+        alphaStarLongIndx[l * nObs + r] = which(XpRE[l * nObs + r], alphaLevelIndx, nDetRE);
+        for (i = 0; i < N; i++) {
+          alphaStarObs[i * nObs + r] += alphaStar[i * nDetRE + alphaStarLongIndx[l * nObs + r]];
         }
       }
     }
@@ -581,7 +587,11 @@ extern "C" {
 	    // of a random effect. 
             for (j = 0; j < J; j++) {
               if (XRE[betaStarIndx[l] * J + j] == betaLevelIndx[l]) {
-                tmp_one[0] += kappaOcc[j] - (F77_NAME(ddot)(&pOcc, &X[j], &J, &beta[i], &N) + betaStarSites[i * J + j] - betaStar[i * nOccRE + l]) * omegaOcc[j];
+                tmp_02 = 0.0;
+                for (ll = 0; ll < pOccRE; ll++) {
+                  tmp_02 += betaStar[i * nOccRE + betaStarLongIndx[ll * J + j]];
+	        } 
+                tmp_one[0] += kappaOcc[j] - (F77_NAME(ddot)(&pOcc, &X[j], &J, &beta[i], &N) + tmp_02 - betaStar[i * nOccRE + l]) * omegaOcc[j];
 	        tmp_0 += omegaOcc[j];
 	      }
             }
@@ -597,7 +607,7 @@ extern "C" {
           zeros(&betaStarSites[i * J], J);
           for (j = 0; j < J; j++) {
             for (l = 0; l < pOccRE; l++) {
-              betaStarSites[i * J + j] += betaStar[i * nOccRE + which(XRE[l * J + j], betaLevelIndx, nOccRE)];
+              betaStarSites[i * J + j] += betaStar[i * nOccRE + betaStarLongIndx[l * J + j]];
             }
           }
 	}
@@ -616,7 +626,11 @@ extern "C" {
 	    tmp_0 = 0.0;
             for (r = 0; r < nObs; r++) {
               if ((z[zLongIndx[r] * N + i] == 1.0) && (XpRE[alphaStarIndx[l] * nObs + r] == alphaLevelIndx[l])) {
-                tmp_one[0] += kappaDet[r] - (F77_NAME(ddot)(&pDet, &Xp[r], &nObs, &alpha[i], &N) + alphaStarObs[i * nObs + r] - alphaStar[i * nDetRE + l]) * omegaDet[r];
+                tmp_02 = 0.0;
+                for (ll = 0; ll < pDetRE; ll++) {
+                  tmp_02 += alphaStar[i * nDetRE + alphaStarLongIndx[ll * nObs + r]];
+	        } 
+                tmp_one[0] += kappaDet[r] - (F77_NAME(ddot)(&pDet, &Xp[r], &nObs, &alpha[i], &N) + tmp_02 - alphaStar[i * nDetRE + l]) * omegaDet[r];
 		  tmp_0 += omegaDet[r];
 	      }
 	    }
@@ -631,7 +645,7 @@ extern "C" {
           // Update the RE sums for the current species
           for (r = 0; r < nObs; r++) {
             for (l = 0; l < pDetRE; l++) {
-              alphaStarObs[i * nObs + r] += alphaStar[i * nDetRE + which(XpRE[l * nObs + r], alphaLevelIndx, nDetRE)]; 
+              alphaStarObs[i * nObs + r] += alphaStar[i * nDetRE + alphaStarLongIndx[l * nObs + r]]; 
             }
           }
         }
